<?php

$installer = $this;

$installer->startSetup();

$installer->run("

DROP TABLE IF EXISTS {$this->getTable('affiliate_referer')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_payment_paypal')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_payment')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_transaction')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_banner_value')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_banner')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_account_value')};
DROP TABLE IF EXISTS {$this->getTable('affiliate_account')};

CREATE TABLE {$this->getTable('affiliate_account')}(
  `account_id` int(10) unsigned NOT NULL auto_increment,
  `customer_id` int(10) unsigned NOT NULL,
  `address_id` int(10) unsigned NOT NULL default '0',
  `identify_code` varchar(63) NOT NULL default '',
  `name` varchar(255) NOT NULL default '',
  `email` varchar(255) NOT NULL default '',
  `balance` decimal(12,4) NOT NULL default '0',
  `total_commission_received` decimal(12,4) NOT NULL default '0',
  `total_paid` decimal(12,4) NOT NULL default '0',
  `total_clicks` int(11) NOT NULL default '0',
  `unique_clicks` int(11) NOT NULL default '0',
  `paypal_email` varchar(255) NOT NULL default '',
  `created_time` datetime NOT NULL default '0000-00-00 00:00:00',
  `status` tinyint(1) NOT NULL default '2',
  `approved` tinyint(1) NOT NULL default '2',
  UNIQUE(`customer_id`),
  UNIQUE(`identify_code`),
  INDEX(`customer_id`),
  FOREIGN KEY (`customer_id`) REFERENCES {$this->getTable('customer/entity')}(`entity_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`account_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_account_value')}(
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `account_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned  NOT NULL,
  `attribute_code` varchar(63) NOT NULL default '',
  `value` text NOT NULL,
  UNIQUE(`account_id`,`store_id`,`attribute_code`),
  INDEX (`account_id`),
  INDEX (`store_id`),
  FOREIGN KEY (`account_id`) REFERENCES {$this->getTable('affiliate_account')} (`account_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  FOREIGN KEY (`store_id`) REFERENCES {$this->getTable('core/store')} (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`value_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_banner')}(
  `banner_id` int(10) unsigned NOT NULL auto_increment,
  `title` varchar(255) NOT NULL default '',
  `type_id` tinyint(1) NOT NULL default '1',
  `source_file` varchar(255) NOT NULL default '',
  `width` int(11) NOT NULL,
  `height` int(11) NOT NULL,
  `link` varchar(255) NOT NULL,
  `status` tinyint(1) NOT NULL default '1',
  PRIMARY KEY (`banner_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_banner_value')}(
  `value_id` int(10) unsigned NOT NULL auto_increment,
  `banner_id` int(10) unsigned NOT NULL,
  `store_id` smallint(5) unsigned  NOT NULL,
  `attribute_code` varchar(63) NOT NULL default '',
  `value` text NOT NULL,
  UNIQUE(`banner_id`,`store_id`,`attribute_code`),
  INDEX (`banner_id`),
  INDEX (`store_id`),
  FOREIGN KEY (`banner_id`) REFERENCES {$this->getTable('affiliate_banner')} (`banner_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  FOREIGN KEY (`store_id`) REFERENCES {$this->getTable('core/store')} (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`value_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_transaction')}(
  `transaction_id` int(10) unsigned NOT NULL auto_increment,
  `account_id` int(10) unsigned NOT NULL,
  `account_name` varchar(255) NOT NULL default '',
  `account_email` varchar(255)  NOT NULL,
  `customer_id` int(10) unsigned  NOT NULL,
  `customer_email` varchar(255)  NOT NULL,
  `order_id` int(10) unsigned  NOT NULL,
  `order_number` varchar(50) default '',
  `order_item_ids` text default '',
  `order_item_names` text default '',
  `total_amount` decimal(12,4) NOT NULL default '0',
  `commission` decimal(12,4) NOT NULL default '0',
  `discount` decimal(12,4) NOT NULL default '0',
  `created_time` datetime NULL,
  `status` tinyint(1) NOT NULL default '1',
  `store_id` smallint(5) unsigned  NOT NULL,
  INDEX(`account_id`),
  INDEX(`store_id`),
  FOREIGN KEY (`account_id`) REFERENCES {$this->getTable('affiliate_account')} (`account_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  FOREIGN KEY (`store_id`) REFERENCES {$this->getTable('core/store')} (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`transaction_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_payment')}(
  `payment_id` int(10) unsigned NOT NULL auto_increment,
  `account_id` int(10) unsigned  NOT NULL,
  `account_name` varchar(255) NOT NULL,
  `account_email` varchar(255) NOT NULL,
  `payment_method` varchar(63) NOT NULL default '',
  `amount` decimal(12,4) NOT NULL default '0',
  `fee` decimal(12,4) NOT NULL default '0',
  `request_time` datetime NULL,
  `status` tinyint(1) NOT NULL default '1',
  `description` text NOT NULL,
  `store_ids` text NOT NULL,
  `is_request` tinyint(1) NOT NULL default '1',
  `is_payer_fee` tinyint(1) NOT NULL default '1',
  INDEX(`account_id`),
  FOREIGN KEY (`account_id`) REFERENCES {$this->getTable('affiliate_account')} (`account_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`payment_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_payment_paypal')}(
  `payment_paypal_id` int(10) unsigned NOT NULL auto_increment,
  `payment_id` int(10) unsigned NOT NULL,
  `email` varchar(255) NOT NULL default '',
  `transaction_id` varchar(255) NOT NULL default '',
  `description` text NOT NULL default '',
  INDEX(`payment_id`),
  FOREIGN KEY (`payment_id`) REFERENCES {$this->getTable('affiliate_payment')} (`payment_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`payment_paypal_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE {$this->getTable('affiliate_referer')}(
  `referer_id` int(10) unsigned NOT NULL auto_increment,
  `account_id` int(10) unsigned NOT NULL,
  `referer` varchar(255) NOT NULL default '',
  `url_path` varchar(255) NOT NULL default '/',
  `total_clicks` int(11) NOT NULL default '0',
  `unique_clicks` int(11) NOT NULL default '0',
  `ip_list` longtext NOT NULL default '',
  `store_id` smallint(5) unsigned  NOT NULL,
  INDEX(`account_id`),
  INDEX(`store_id`),
  FOREIGN KEY (`account_id`) REFERENCES {$this->getTable('affiliate_account')} (`account_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  FOREIGN KEY (`store_id`) REFERENCES {$this->getTable('core/store')} (`store_id`) ON DELETE CASCADE ON UPDATE CASCADE,
  PRIMARY KEY (`referer_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


");
if (version_compare(Mage::getVersion(), '1.4.1.0', '>=')) {
    $installer->getConnection()->addColumn($this->getTable('sales/order'), 'affiliate_discount', ' decimal(12,4) default NULL');
    $installer->getConnection()->addColumn($this->getTable('sales/order'), 'base_affiliate_discount', ' decimal(12,4) default NULL');
}

$content = '
<p>Welcome to the Affiliate Programs. The affiliate program provides a unique way for your website to link to millions products while earning commissions on qualifying sales. Apply now to join the program and earn commissions, it is FREE!</p>
<h3>Real-Time Reporting</h3>
<p>Login daily to check our sales, traffic, account balance and see how your banners are performing.</p>';

//$cmsPage = array(
//    'title'           => Mage::helper('affiliate')->__('Affiliate'),
//    'identifier'      => 'affiliate-home',
//    'content_heading' => Mage::helper('affiliate')->__('Welcome to our affiliate program!'),
//    'content'         => $content,
//    'is_active'       => 1,
//    'sort_order'      => 0,
//    'stores'          => array(0),
//    'root_template'   => 'two_columns_left'
//);
//
//Mage::getModel('cms/page')->setData($cmsPage)->save();

$installer->endSetup(); 
